/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*

Input:
- an integer "position" pos of dimension D
- for each d=0,1, ...,D the min and max possible value

Output:
- an integer position post "as near as possible" of pos, 
	but with all components different
	
Error message: 
may not find any acceptable position


 * main.c
 * Copyright (C) Maurice Clerc 2011 <Maurice.Clerc@WriteMe.com>
 * 
 * All_different is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * All_different is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdio.h>
#define dMax 10

// Structures
struct position {
int size;
int x[dMax];
};

// Subprograms
struct position	all_different(struct position pos, int min[], int max[]);
void display_position(struct position pos);

// Main program
int main()
{
	int d;
	int max[dMax];
	int min[dMax];
	struct position pos;
	struct position post;
	
	// Data
	pos.size=4;
	
	pos.x[0]=5;
	pos.x[1]=5;
	pos.x[2]=3;
	pos.x[3]=4;
	for(d=0;d<pos.size;d++) {min[d]=1; max[d]=5;}
	
	// Modify the position
	post=all_different(pos, min, max);
	
	// Results
	printf("Initial position: ");
	display_position(pos);	
	
	printf("\nModified position: ");
	display_position(post);

	return (0);
}
// ----------------------------------------------------------------------------- ALL_DIFFERENT
struct position	all_different(struct position pos,int min[], int max[])
{
/*
 Modify the position so that all components are different
 Useful for some combinatorial problems, like Knapsack or Fifty-fifty
*/

int				d,d1;
int				k;
float			max_loop;
float			nloop;
struct position	post;


post=pos;


for (d=1;d<pos.size;d++) // For each component ...
{
	max_loop=max[d]-min[d]+1;
	k=1;
	nloop=0;
	loop1:
	//... if component different of all previous ones, OK, ...
	for (d1=0;d1<d;d1++)
	{
		if (post.x[d]==post.x[d1]) // ... else, modify slightly and retry
		{
			loop2:
			post.x[d]=pos.x[d]-k;
			if (k<0) {k=-k+1;} else {k=-k;}
			if (post.x[d]<min[d]) goto loop2;
			if (post.x[d]>max[d]) goto loop2;
				nloop=nloop+1;
				if (nloop==max_loop)
				{
					printf("\n WARNING. Some components are equal (%i and %i) Impossible to modify this position:",d1,d);
					display_position(post);
					return post;
				}
				goto loop1;
		}
	}
}
return post;
}
// ----------------------------------------------------------------------------- DISPLAY_POSITION
void display_position(struct position pos)
{
int				d;
int				ncar=0;

//printf("\n position size %i\n ",pos.size);
for (d=0;d<pos.size;d++)
{
	printf("%i ",pos.x[d]);
	ncar=ncar+10;
	if (ncar>75)
	{
		ncar=0;
		printf("\n");
	}
}

}